# Generated by Django 4.2.4 on 2023-08-25 04:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('import_export_extensions', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='exportjob',
            name='export_status',
            field=models.CharField(choices=[('CREATED', 'Created'), ('EXPORTING', 'Exporting'), ('EXPORT_ERROR', 'Export Error'), ('EXPORTED', 'Exported'), ('CANCELLED', 'Cancelled')], default='CREATED', max_length=20, verbose_name='Job status'),
        ),
    ]
